package com.chelvc.framework.wechat;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.common.util.NumberUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.wechat.context.WechatContextHolder;
import com.google.common.collect.Lists;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.NonNull;
import lombok.experimental.SuperBuilder;

/**
 * 微信订单信息
 *
 * @author Woody
 * @date 2025/6/9
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
public class WechatOrder implements Serializable {
    /**
     * 微信订单号
     */
    private String id;

    /**
     * 商户订单号
     */
    private String code;

    /**
     * 支付方式
     */
    private String mode;

    /**
     * 付款银行
     */
    private String back;

    /**
     * 应用标识
     */
    private String appid;

    /**
     * 商户标识
     */
    private String mchid;

    /**
     * 用户标识
     */
    private String openid;

    /**
     * 终端设备号
     */
    private String device;

    /**
     * 交易状态
     */
    private String status;

    /**
     * 订单完成时间
     */
    private Date datetime;

    /**
     * 现金支付币种
     */
    private String cashType;

    /**
     * 标价币种
     */
    private String amountType;

    /**
     * 附加数据
     */
    private String attachment;

    /**
     * 现金支付金额
     */
    private BigDecimal cashAmount;

    /**
     * 标价金额
     */
    private BigDecimal totalAmount;

    /**
     * 应结订单金额
     */
    private BigDecimal settleAmount;

    /**
     * 代金券金额
     */
    private BigDecimal couponAmount;

    /**
     * 代金券列表
     */
    private List<WechatCoupon> coupons;

    /**
     * 构建订单实例
     *
     * @param map 订单信息键/值映射表
     * @return 订单对象实例
     */
    public static WechatOrder of(@NonNull Map<String, String> map) {
        return of(new WechatOrder(), map);
    }

    /**
     * 构建订单实例
     *
     * @param order 订单对象实例
     * @param map   订单信息键/值映射表
     * @param <T>   订单对象类型
     * @return 订单对象实例
     */
    public static <T extends WechatOrder> T of(@NonNull T order, @NonNull Map<String, String> map) {
        order.setId(map.get("transaction_id"));
        order.setCode(map.get("out_trade_no"));
        order.setMode(map.get("trade_type"));
        order.setBack(map.get("bank_type"));
        order.setAppid(map.get("appid"));
        order.setMchid(map.get("mch_id"));
        order.setOpenid(map.get("openid"));
        order.setDevice(map.get("device_info"));
        order.setStatus(map.get("trade_state"));
        order.setDatetime(DateUtils.number2datetime(map.get("time_end")));
        order.setCashType(map.get("cash_fee_type"));
        order.setAmountType(map.get("fee_type"));
        order.setAttachment(map.get("attach"));
        order.setCashAmount(WechatContextHolder.string2amount(map.get("cash_fee")));
        order.setTotalAmount(WechatContextHolder.string2amount(map.get("total_fee")));
        order.setSettleAmount(WechatContextHolder.string2amount(map.get("settlement_total_fee")));
        order.setCouponAmount(WechatContextHolder.string2amount(map.get("coupon_fee")));

        // 初始化优惠卷信息
        Integer couponCount = ObjectUtils.ifNull(map.get("coupon_count"), Integer::valueOf);
        if (NumberUtils.isGreaterZero(couponCount)) {
            List<WechatCoupon> coupons = Lists.newArrayListWithCapacity(couponCount);
            for (int i = 0; i < couponCount; i++) {
                String id = map.get("coupon_id_" + i);
                String type = map.get("coupon_type_" + i);
                BigDecimal amount = WechatContextHolder.string2amount(map.get("coupon_fee_" + i));
                coupons.add(WechatCoupon.builder().id(id).type(type).amount(amount).build());
            }
            order.setCoupons(Collections.unmodifiableList(coupons));
        } else {
            order.setCoupons(Collections.emptyList());
        }
        return order;
    }
}
